@@ -13,7 +13,7 @@
     grid
     grid_surface
     lines
-    multiprocessing
+    multi_processing
     organize
     property
     rq_import
@@ -28,6 +28,6 @@
 
 import logging
 
-__version__ = "3.7.3"  # Set at build time
+__version__ = "4.5.0"  # Set at build time
 log = logging.getLogger(__name__)
 log.info(f"Imported resqpy version {__version__}")
