@@ -6,44 +6,49 @@ log = logging.getLogger(__name__)
 
 import math as maths
 import numpy as np
+import warnings
 
 import resqpy.olio.intersection as meet
-from resqpy.lines._common import _BasePolyline, load_hdf5_array, tangents, spline
+import resqpy.lines
 import resqpy.olio.point_inclusion as pip
 import resqpy.olio.uuid as bu
 import resqpy.olio.vector_utilities as vu
 import resqpy.olio.write_hdf5 as rwh5
 import resqpy.olio.xml_et as rqet
+import resqpy.lines._common as rql_c
 from resqpy.olio.xml_namespaces import curly_namespace as ns
 
 
-class Polyline(_BasePolyline):
+class Polyline(rql_c._BasePolyline):
     """Class for RESQML polyline representation."""
 
     resqml_type = 'PolylineRepresentation'
 
-    def __init__(self,
-                 parent_model,
-                 uuid = None,
-                 set_bool = None,
-                 set_coord = None,
-                 set_crs = None,
-                 title = None,
-                 rep_int_root = None,
-                 originator = None,
-                 extra_metadata = None):
-        """Initialises a new PolylineRepresentation object.
+    def __init__(
+            self,
+            parent_model,
+            uuid = None,
+            set_bool = None,  #: DEPRECATED
+            set_coord = None,
+            set_crs = None,
+            is_closed = None,
+            title = None,
+            rep_int_root = None,
+            originator = None,
+            extra_metadata = None):
+        """Initialises a new polyline object.
 
         arguments:
             parent_model (model.Model object): the model which the new PolylineRepresentation belongs to
             uuid (uuid.UUID, optional): the uuid of an existing RESQML PolylineRepresentation from which
                 to initialise the resqpy Polyline
-            set_bool (boolean, optional): if True, a new polyline created from coordinates is flagged as
-                a closed polyline (polygon); ignored if uuid is not None
-            set_coord (numpy array of shape (..., 3), optional): an ordered set of xyz values used to define
-                a new polyline; ignored if uuid is not None
+            set_bool (boolean, optional): DEPRECATED: synonym for is_closed argument
+            set_coord (numpy float array, optional): an ordered set of xyz values used to define a new polyline;
+                last dimension of array must have extent 3; ignored if uuid is not None
             set_crs (uuid.UUID, optional): the uuid of a crs to be used when initialising from coordinates;
                 ignored if uuid is not None
+            is_closed (boolean, optional): if True, a new polyline created from coordinates is flagged as
+                a closed polyline (polygon); ignored if uuid is not None
             title (str, optional): the citation title to use for a new polyline;
                 ignored if uuid is not None
             rep_int_root
@@ -53,13 +58,17 @@ class Polyline(_BasePolyline):
                 ignored if uuid is not None
 
         returns:
-            the newly instantiated Polyline object
+            the newly instantiated polyline object
 
         :meta common:
         """
 
         self.model = parent_model
-        self.isclosed = set_bool
+        if set_bool is not None:
+            warnings.warn('DEPRECATED: use is_closed argument instead of set_bool, in Polyline initialisation')
+            if is_closed is None:
+                is_closed = set_bool
+        self.isclosed = is_closed
         self.nodepatch = None
         self.crs_uuid = set_crs
         self.coordinates = None
@@ -71,7 +80,7 @@ class Polyline(_BasePolyline):
                          originator = originator,
                          extra_metadata = extra_metadata)
 
-        if self.root is None and all(i is not None for i in [set_bool, set_coord, set_crs, title]):
+        if self.root is None and all(i is not None for i in [is_closed, set_coord, set_crs, title]):
             # Using data from a polyline set
             assert set_coord.ndim > 1 and 2 <= set_coord.shape[-1] <= 3
             # allow for x,y or x,y,z incoming coordinates but use x,y,z internally
@@ -111,7 +120,7 @@ class Polyline(_BasePolyline):
 
         points_node = rqet.find_tag(geometry_node, 'Points')
         assert points_node is not None  # Required field
-        load_hdf5_array(self, points_node, 'coordinates', tag = 'Coordinates')
+        rql_c.load_hdf5_array(self, points_node, 'coordinates', tag = 'Coordinates')
 
         self.nodepatch = (rqet.find_tag_int(patch_node, 'PatchIndex'), rqet.find_tag_int(patch_node, 'Count'))
         assert not any(map(lambda x: x is None, self.nodepatch))  # Required fields - assert neither are None
@@ -153,7 +162,7 @@ class Polyline(_BasePolyline):
 
         polyline = cls(original.model,
                        set_crs = original.crs_uuid,
-                       set_bool = original.isclosed,
+                       is_closed = original.isclosed,
                        title = title if title else original.title,
                        originator = originator if originator else original.originator,
                        extra_metadata = extra_metadata)
@@ -201,7 +210,7 @@ class Polyline(_BasePolyline):
 
         polyline = cls(original.model,
                        set_crs = original.crs_uuid,
-                       set_bool = False,
+                       is_closed = False,
                        title = title if title else original.title,
                        originator = originator if originator else original.originator,
                        extra_metadata = extra_metadata)
@@ -243,7 +252,47 @@ class Polyline(_BasePolyline):
             theta = i * 2.0 * maths.pi / float(n)
             coords[i] = np.array((radius * maths.sin(theta), radius * maths.cos(theta), 0.0), dtype = float) +  \
                         np.array(centre_xyz, dtype = float)
-        polyline = cls(model, set_bool = True, set_coord = coords, set_crs = crs_uuid, title = title)
+        polyline = cls(model, is_closed = True, set_coord = coords, set_crs = crs_uuid, title = title)
+
+        return polyline
+
+    @classmethod
+    def convex_hull_from_closed_polyline(cls, original, title, mode = 'crossing'):
+        """Returns a new closed convex polyline being the convex hull of an original closed polyline."""
+
+        assert original.isclosed
+        assert len(original.coordinates) >= 3
+        if not title:
+            title = f'{original.title} convex hull'
+
+        if len(original.coordinates) == 3 or original.is_convex():
+            coords = original.coordinates
+        else:
+            oc = original.coordinates
+            while True:
+                ol = len(oc)
+                assert ol >= 3
+                hull_mask = np.zeros(ol, dtype = bool)
+                for i in range(ol):
+                    if i == 0:
+                        coords = oc[1:]
+                    elif i == ol - 1:
+                        coords = oc[:-1]
+                    else:
+                        coords = np.concatenate((oc[:i], oc[i + 1:]))
+                    if mode == 'crossing':
+                        inside = pip.pip_cn(oc[i], coords)
+                    else:
+                        inside = pip.pip_wn(oc[i], coords)
+                    hull_mask[i] = not inside
+                if np.all(hull_mask):
+                    coords = oc
+                    break
+                oc = oc[hull_mask]
+            assert len(coords) >= 3
+
+        polyline = cls(original.model, is_closed = True, set_coord = coords, set_crs = original.crs_uuid, title = title)
+        assert polyline.is_convex(trust_metadata = False)
 
         return polyline
 
@@ -315,8 +364,8 @@ class Polyline(_BasePolyline):
         """Returns bool array, True where p is inside closed polygon, in xy plane, otherwise False.
 
         arguments:
-           p_array (numpy float array of shape (..., 2 or 3)): an array of points, each of which is
-              tested for inclusion against the closed polygon
+           p_array (numpy float array): an array of points, each of which is tested for inclusion against
+              the closed polygon; the final axis of the array must have extent 2 or 3
 
         returns:
            numpy bool array of shape p_array.shape[:-1], set True for those points which are inside
@@ -443,7 +492,7 @@ class Polyline(_BasePolyline):
            or None, None, None if no intersection found
 
         note:
-           'first' primariliy refers to the ordering of segments in this polyline
+           first primariliy refers to the ordering of segments in this polyline
         """
 
         seg_count = len(self.coordinates) - 1
@@ -546,7 +595,7 @@ class Polyline(_BasePolyline):
         return crossings
 
     def normalised_xy(self, x, y, mode = 'square'):
-        """Returns a normalised x',y' pair (in range 0..1) being point x,y under mapping from convex polygon.
+        """Returns a normalised x,y pair (in range 0..1) being point x,y under mapping from convex polygon.
 
         arguments:
             x, y (floats): location of a point inside the polyline, which must be closed and project to a
@@ -554,7 +603,7 @@ class Polyline(_BasePolyline):
             mode (string): which mapping algorithm to use, one of: 'square', 'circle', or 'perimeter'
 
         returns:
-            x', y' (floats, each in range 0..1) being the normalised representation of point x,y
+            xn, yn (floats, each in range 0..1) being the normalised representation of point x,y
 
         notes:
             this method is the inverse of denormalised_xy(), as long as a consistent mode is selected;
@@ -689,7 +738,7 @@ class Polyline(_BasePolyline):
     def tangent_vectors(self):
         """Returns a numpy array of unit length tangent vectors, one for each coordinate in the line."""
 
-        return tangents(self.coordinates)
+        return rql_c.tangents(self.coordinates)
 
     def splined(self,
                 tangent_weight = 'square',
@@ -703,12 +752,12 @@ class Polyline(_BasePolyline):
         :meta common:
         """
 
-        spline_coords = spline(self.coordinates,
-                               tangent_weight = tangent_weight,
-                               min_subdivisions = min_subdivisions,
-                               max_segment_length = max_segment_length,
-                               max_degrees_per_knot = max_degrees_per_knot,
-                               closed = self.isclosed)
+        spline_coords = rql_c.spline(self.coordinates,
+                                     tangent_weight = tangent_weight,
+                                     min_subdivisions = min_subdivisions,
+                                     max_segment_length = max_segment_length,
+                                     max_degrees_per_knot = max_degrees_per_knot,
+                                     closed = self.isclosed)
 
         if not title:
             title = self.title
@@ -716,7 +765,7 @@ class Polyline(_BasePolyline):
             rep_int_root = self.rep_int_root  # todo: check whether it is legal to have 2 representations for 1 interpretation
 
         return Polyline(self.model,
-                        set_bool = self.isclosed,
+                        is_closed = self.isclosed,
                         set_coord = spline_coords,
                         set_crs = self.crs_uuid,
                         title = title,
@@ -754,9 +803,9 @@ class Polyline(_BasePolyline):
                    add_relationships = True,
                    title = None,
                    originator = None):
-        """Create xml from polyline.
+        """Create xml for polyline and optionally adds as part to model.
 
-        args:
+        arguments:
             ext_uuid: the uuid of the hdf5 external part
 
         :meta common:
@@ -776,8 +825,10 @@ class Polyline(_BasePolyline):
             rep_int = self.rep_int_root
             if "FaultInterpretation" in str(rqet.content_type(rep_int)):
                 content_type = 'obj_FaultInterpretation'
-            else:
+            elif "HorizonInterpretation" in str(rqet.content_type(rep_int)):
                 content_type = 'obj_HorizonInterpretation'
+            else:
+                content_type = 'obj_BoundaryFeatureInterpretation'
             self.model.create_ref_node('RepresentedInterpretation',
                                        rqet.citation_title_for_node(rep_int),
                                        rep_int.attrib['uuid'],
