@@ -32,6 +32,11 @@ def _create_tree_if_none(model):
 def _change_uuid_in_supporting_representation_reference(model, node, old_uuid, new_uuid, new_title = None):
     """Look for supporting representation reference using the old_uuid and replace with the new_uuid."""
 
+    if isinstance(old_uuid, str):
+        old_uuid = bu.uuid_from_string(old_uuid)
+    if isinstance(new_uuid, str):
+        new_uuid = bu.uuid_from_string(new_uuid)
+
     ref_node = rqet.find_tag(node, 'SupportingRepresentation')
     if ref_node is None:
         return False
@@ -40,6 +45,20 @@ def _change_uuid_in_supporting_representation_reference(model, node, old_uuid, n
         return False
     if not bu.matching_uuids(uuid_node.text, old_uuid):
         return False
+
+    uuid_node_int = rqet.uuid_for_part_root(node).int
+    relations = model.uuid_rels_dict[uuid_node_int]
+    if old_uuid.int in relations[0]:
+        relations[0].remove(old_uuid.int)
+    relations[0].add(new_uuid.int)
+
+    relations = model.uuid_rels_dict[old_uuid.int]
+    if uuid_node_int in relations[1]:
+        relations[1].remove(uuid_node_int)
+
+    relations = model.uuid_rels_dict[new_uuid.int]
+    relations[1].add(uuid_node_int)
+
     uuid_node.text = str(new_uuid)
     if new_title:
         title_node = rqet.find_tag(ref_node, 'Title')
@@ -87,10 +106,11 @@ def _referenced_node(model, ref_node, consolidate = False):
     if uuid is None:
         return None
     # return model.root_for_part(model.parts_list_of_type(type_of_interest = content_type, uuid = uuid))
-    if consolidate and model.consolidation is not None and uuid in model.consolidation.map:
-        resident_uuid = model.consolidation.map[uuid]
-        if resident_uuid is None:
+    if consolidate and model.consolidation is not None and uuid.int in model.consolidation.map:
+        resident_uuid_int = model.consolidation.map[uuid.int]
+        if resident_uuid_int is None:
             return None
+        resident_uuid = bu.uuid_for_int(resident_uuid_int)
         node = model.root_for_part(model.part_for_uuid(resident_uuid))
         if node is not None:
             # patch resident uuid and title into ref node!
@@ -107,7 +127,7 @@ def _referenced_node(model, ref_node, consolidate = False):
     return node
 
 
-def _create_ref_node(flavour, title, uuid, content_type = None, root = None):
+def _create_ref_node(model, flavour, title, uuid, content_type = None, root = None):
     """Create a reference node, optionally add to root."""
 
     assert uuid is not None
@@ -134,7 +154,9 @@ def _create_ref_node(flavour, title, uuid, content_type = None, root = None):
         ct_node.text = 'application/x-resqml+xml;version=2.0;type=' + content_type
 
     if not title:
-        title = '(title unavailable)'
+        title = model.title(uuid = uuid)
+        if title is None:
+            title = 'untitled'
     title_node = rqet.SubElement(ref_node, ns['eml'] + 'Title')
     title_node.set(ns['xsi'] + 'type', ns['eml'] + 'DescriptionString')
     title_node.text = title
@@ -182,8 +204,8 @@ def _create_rels_part(model):
 def _create_citation(root = None, title = '', originator = None):
     """Creates a citation xml node and optionally appends as a child of root."""
 
-    if not title:
-        title = '(no title)'
+    if title is None:
+        title = ''
 
     citation = rqet.Element(ns['eml'] + 'Citation')
     citation.set(ns['xsi'] + 'type', ns['eml'] + 'Citation')
@@ -272,18 +294,22 @@ def _create_crs_reference(model, root = None, crs_uuid = None):
     assert crs_uuid is not None
     crs_root = model.root_for_uuid(crs_uuid)
     assert crs_root is not None
+    crs_type = model.type_of_uuid(crs_uuid)
+    assert crs_type in ['obj_LocalDepth3dCrs', 'obj_LocalTime3dCrs']
 
-    return _create_ref_node('LocalCrs',
+    return _create_ref_node(model,
+                            'LocalCrs',
                             rqet.find_nested_tags_text(crs_root, ['Citation', 'Title']),
                             crs_uuid,
-                            content_type = 'obj_LocalDepth3dCrs',
+                            content_type = crs_type,
                             root = root)
 
 
-def _create_md_datum_reference(md_datum_root, root = None):
+def _create_md_datum_reference(model, md_datum_root, root = None):
     """Creates a node refering to an existing measured depth datum and optionally adds as child of root."""
 
-    return _create_ref_node('MdDatum',
+    return _create_ref_node(model,
+                            'MdDatum',
                             rqet.find_nested_tags_text(md_datum_root, ['Citation', 'Title']),
                             bu.uuid_from_string(md_datum_root.attrib['uuid']),
                             content_type = 'obj_MdDatum',
@@ -339,7 +365,7 @@ def _create_hdf5_ext(model,
     return ext
 
 
-def _create_hdf5_dataset_ref(hdf5_uuid, object_uuid, group_tail, root, title = 'Hdf Proxy'):
+def _create_hdf5_dataset_ref(model, hdf5_uuid, object_uuid, group_tail, root, title = 'Hdf Proxy'):
     """Creates a pair of nodes referencing an hdf5 dataset (array) and adds to root."""
 
     assert root is not None
@@ -356,7 +382,7 @@ def _create_hdf5_dataset_ref(hdf5_uuid, object_uuid, group_tail, root, title = '
     path_node.text = hdf5_path
     root.append(path_node)
 
-    _create_ref_node('HdfProxy', title, hdf5_uuid, content_type = 'obj_EpcExternalPartReference', root = root)
+    _create_ref_node(model, 'HdfProxy', title, hdf5_uuid, content_type = 'obj_EpcExternalPartReference', root = root)
 
     return path_node
 
@@ -385,7 +411,12 @@ def _create_supporting_representation(model,
         if not title:
             title = 'supporting representation'
 
-    return _create_ref_node('SupportingRepresentation', title, support_uuid, content_type = content_type, root = root)
+    return _create_ref_node(model,
+                            'SupportingRepresentation',
+                            title,
+                            support_uuid,
+                            content_type = content_type,
+                            root = root)
 
 
 def _create_source(source, root = None):
@@ -409,7 +440,8 @@ def _create_source(source, root = None):
     return emd_node
 
 
-def _create_patch(p_uuid,
+def _create_patch(model,
+                  p_uuid,
                   ext_uuid = None,
                   root = None,
                   patch_index = 0,
@@ -455,7 +487,7 @@ def _create_patch(p_uuid,
     outer_values_node.set(ns['xsi'] + 'type', ns['resqml2'] + hdf5_type)  # may also be constant array type
     outer_values_node.text = rqet.null_xml_text
 
-    if discrete:
+    if discrete and const_value is None:
         if null_value is None:
             if str(xsd_type).startswith('u'):
                 null_value = 4294967295  # 2^32 - 1, used as default even for 64 bit data!
@@ -471,7 +503,7 @@ def _create_patch(p_uuid,
         inner_values_node.set(ns['xsi'] + 'type', ns['eml'] + 'Hdf5Dataset')
         inner_values_node.text = rqet.null_xml_text
 
-        _create_hdf5_dataset_ref(ext_uuid, p_uuid, f'{hdf_path_tail}{patch_index}', root = inner_values_node)
+        _create_hdf5_dataset_ref(model, ext_uuid, p_uuid, f'{hdf_path_tail}{patch_index}', root = inner_values_node)
 
     else:
 
@@ -565,3 +597,17 @@ def _create_reciprocal_relationship(model, node_a, rel_type_a, node_b, rel_type_
                   id_str(uuid_a))  # NB: fesapi prefixes uuid with _ for some rels only (where uuid starts with a digit)
         rel_b.set('Type', ns_url['rels_ext'] + rel_type_b)
         rel_b.set('Target', part_name_a)
+
+    if "EpcExternalPart" in rel_part_name_a or "EpcExternalPart" in rel_part_name_b:
+        return
+
+    rel_uuid_a_int = rel_uuid_a.int
+    rel_uuid_b_int = rel_uuid_b.int
+
+    value = model.uuid_rels_dict.get(rel_uuid_a_int)
+    if value is not None and rel_uuid_b_int not in value[0] and rel_uuid_b_int not in value[1]:
+        value[2].add(rel_uuid_b_int)
+
+    value = model.uuid_rels_dict.get(rel_uuid_b_int)
+    if value is not None and rel_uuid_a_int not in value[0] and rel_uuid_a_int not in value[1]:
+        value[2].add(rel_uuid_a_int)
