@@ -1,14 +1,12 @@
 """intersection.py: functions to test whether lines intersect with planes."""
 
-version = '5th November 2020'
-
 import logging
 
 log = logging.getLogger(__name__)
 
-from typing import Union
 import numpy as np
 from numba import njit  # type: ignore
+from typing import Union
 
 import resqpy.olio.vector_utilities as vec
 
@@ -89,17 +87,17 @@ def line_triangle_intersect_numba(
 ) -> Union[None, np.ndarray]:
     """Find the intersection of a line within a triangle in 3D space.
 
-    Args:
+    arguments:
         line_p (np.ndarray): a point on the line.
         line_v (np.ndarray): vector being the direction of the line.
-        triangle (np.ndarray): three corners of the triangle (second index is xyz).
+        triangle (np.ndarray): shape (3, 3); three corners of the triangle (second index is xyz).
         line_segment (bool): if True, returns None if intersection is outwith (line_p .. line_p + line_v).
         l_tol (float, default 0.0): a fraction of the line length to allow for an intersection to be found
             just outside the segment.
         t_tol (float, default 0.0): a fraction of the triangle size to allow for an intersection to be found
             just outside the triangle.
 
-    Returns:
+    returns:
         point (np.ndarray) of intersection of the line within the triangle,
         or None if line is parallel to plane of triangle or intersection with the plane is
         outside the triangle.
@@ -186,7 +184,7 @@ def line_set_triangles_intersects(line_ps, line_vs, triangles, line_segment = Fa
 
     arguments:
        line_ps ((c, 3) numpy array): a point on each of c lines
-       line_vs ((c, 3) numpy array): vectors being the direction of each of the c lines
+       line_vs ((c, 3) numpy array): vectors being the direction of each of the c lines (or 1 common vector)
        triangles ((n, 3, 3) numpy array): three corners of each of the n triangles (final index is xyz)
        line_segment (boolean, default False): if True, each line is treated as a finite segment between
           p and p + v, and only intersections within the segment are included
@@ -203,6 +201,9 @@ def line_set_triangles_intersects(line_ps, line_vs, triangles, line_segment = Fa
     c = line_ps.shape[0]  # number of lines
     n = triangles.shape[0]  # number of triangles
 
+    if line_vs.ndim == 1:  # single common direction vector; for now simply replicate
+        line_vs = np.repeat(np.expand_dims(line_vs, axis = 0), c, axis = 0)
+
     p01s = triangles[:, 1, :] - triangles[:, 0, :]  # p01s has shape (n, 3)
     p02s = triangles[:, 2, :] - triangles[:, 0, :]  # p02s has shape (n, 3)
 
@@ -423,6 +424,7 @@ def line_line_intersect(x1, y1, x2, y2, x3, y3, x4, y4, line_segment = False, ha
 def point_projected_to_line_2d(p, l1, l2):
     """Return the point on the unbounded line passing through l1 & l2 which is closest to point p, in xy plane."""
 
+    # note: result should be at closest point even in the presence of mixed xy & z units (?)
     # create normal vector to l1, l2
     v = l2 - l1
     n = np.array((-v[1], v[0]))
