@@ -1,4 +1,4 @@
-""" Utilities for working with 3D vectors in cartesian space.
+"""Utilities for working with 3D vectors in cartesian space.
 
 note: some of these functions are redundant as they are provided by built-in numpy operations.
 
@@ -6,18 +6,15 @@ a vector is a one dimensional numpy array with 3 elements: x, y, z.
 some functions accept a tuple or list of 3 elements as an alternative to a numpy array.
 """
 
-version = '28th July 2022'
-
 import logging
-from typing import Tuple, Optional
 
 log = logging.getLogger(__name__)
 
 import math as maths
 import numpy as np
 import numba  # type: ignore
-from numba import njit
-import multiprocessing as m_p
+from numba import njit  # type: ignore
+from typing import Tuple, Optional
 
 
 def radians_from_degrees(deg):
@@ -290,7 +287,12 @@ def degrees_difference(a, b):
 
 
 def rotation_matrix_3d_axial(axis, angle):
-    """Returns a rotation matrix which will rotate points about axis (0: x, 1: y, or 2: z) by angle in degrees."""
+    """Returns a rotation matrix which will rotate points about axis (0: x, 1: y, or 2: z) by angle in degrees.
+
+    note:
+       this function follows the mathematical convention: a positive angle results in anti-clockwise rotation
+       when viewed in direction of positive axis
+    """
 
     axis_a = (axis + 1) % 3
     axis_b = (axis_a + 1) % 3
@@ -317,11 +319,14 @@ def no_rotation_matrix():
 def rotation_3d_matrix(xzy_axis_angles):
     """Returns a rotation matrix which will rotate points about 3 axes by angles in degrees."""
 
-    matrix = np.zeros((3, 3))
-    for axis in range(3):
-        matrix[axis, axis] = 1.0
-    for axis in range(3):
-        matrix = np.dot(matrix, rotation_matrix_3d_axial(axis, xzy_axis_angles[axis]))
+    # matrix = np.zeros((3, 3))
+    # for axis in range(3):
+    #     matrix[axis, axis] = 1.0
+    # for axis in range(3):
+    #     matrix = np.dot(matrix, rotation_matrix_3d_axial(axis, xzy_axis_angles[axis]))
+    matrix = rotation_matrix_3d_axial(1, xzy_axis_angles[2])  # about y axis
+    matrix = np.dot(matrix, rotation_matrix_3d_axial(2, xzy_axis_angles[1]))  # about z axis
+    matrix = np.dot(matrix, rotation_matrix_3d_axial(0, xzy_axis_angles[0]))  # about x axis
     return matrix
 
 
@@ -369,9 +374,13 @@ def rotate_xyz_array_around_z_axis(a, target_xy_vector):
 
 
 def unit_vector_from_azimuth_and_inclination(azimuth, inclination):
-    """Returns unit vector with compass bearing of azimuth and inclination off +z axis."""
+    """Returns unit vector with compass bearing of azimuth and inclination off +z axis.
 
-    matrix = rotation_3d_matrix((inclination, azimuth, 0.0))
+    note:
+       assumes a left handed coordinate system with y axis north and x axis east
+    """
+
+    matrix = rotation_3d_matrix((inclination, 180.0 - azimuth, 0.0))
     return rotate_vector(matrix, np.array((0.0, 0.0, 1.0)))
 
 
@@ -412,7 +421,11 @@ def tilt_points(pivot_xyz, azimuth, dip, points):
 
 
 def project_points_onto_plane(plane_xyz, normal_vector, points):
-    """Modifies array of xyz points in situ to project onto a plane defined by a point and normal vector."""
+    """Modifies array of xyz points in situ to project onto a plane defined by a point and normal vector.
+
+    note:
+       implicit xy & z units must be the same
+    """
 
     az = azimuth(normal_vector)
     incl = inclination(normal_vector)
@@ -552,13 +565,16 @@ def points_in_triangles(p, t, da, projection = 'xy', edged = False):
 def point_in_polygon(x, y, polygon):
     """Calculates if a point in within a polygon in 2D.
     
-    Args:
+    arguments:
         x (float): the point's x-coordinate.
         y (float): the point's y-coordinate.
         polygon (np.ndarray): array of the polygon's vertices in 2D.
     
-    Returns:
+    returns:
         inside (bool): True if point is within the polygon, False otherwise.
+
+    note:
+        the polygon is assumed closed, the closing point should not be repeated
     """
     polygon_vertices = len(polygon)
     inside = False
@@ -583,12 +599,12 @@ def point_in_polygon(x, y, polygon):
 def point_in_triangle(x, y, triangle):
     """Calculates if a point in within a triangle in 2D.
 
-    Args:
+    arguments:
         x (float): the point's x-coordinate.
         y (float): the point's y-coordinate.
         triangle (np.ndarray): array of the triangles's vertices in 2D, of shape (3, 2)
 
-    Returns:
+    returns:
         inside (bool): True if point is within the polygon, False otherwise.
     """
     p0x = triangle[0, 0]
@@ -654,22 +670,25 @@ def point_in_triangle(x, y, triangle):
 def points_in_polygon(points: np.ndarray, polygon: np.ndarray, points_xlen: int, polygon_num: int = 0) -> np.ndarray:
     """Calculates which points are within a polygon in 2D.
 
-    Args:
-        points (np.ndarray): array of the points in 2D.
-        polygon (np.ndarray): array of the polygon's vertices in 2D.
-        points_xlen (int): the number of unique x coordinates.
-        polygon_num (int): the polygon number, default is 0.
+    arguments:
+        points (np.ndarray): array of shape (N, 2 or 3), of the points in 2D (xy, any z values are ignored)
+        polygon (np.ndarray): list-like array of the polygon's vertices in 2D
+        points_xlen (int): the original I extent of the now flattened points, use 1 if not applicable
+        polygon_num (int): the polygon number, default is 0, for copying to output
 
-    Returns:
-        polygon_points (np.ndarray): 2D array containing only the points within the polygon,
-            with each row being the polygon number, points y index, and points x index.
+    returns:
+        polygon_points (np.ndarray): list-like 2D array containing only the points within the polygon,
+            with each row being the polygon number (as input), points J index, and points I index
+
+    note:
+        the polygon is assumed closed, the closing point should not be repeated
     """
     polygon_points = np.empty((0, 3), dtype = numba.int32)
     for point_num in numba.prange(len(points)):
         p = point_in_polygon(points[point_num, 0], points[point_num, 1], polygon)
         if p is True:
-            yi, xi = divmod(point_num, points_xlen)
-            polygon_points = np.append(polygon_points, np.array([[polygon_num, yi, xi]], dtype = numba.int32), axis = 0)
+            j, i = divmod(point_num, points_xlen)
+            polygon_points = np.append(polygon_points, np.array([[polygon_num, j, i]], dtype = numba.int32), axis = 0)
 
     return polygon_points
 
@@ -678,13 +697,13 @@ def points_in_polygon(points: np.ndarray, polygon: np.ndarray, points_xlen: int,
 def points_in_triangle(points: np.ndarray, triangle: np.ndarray, points_xlen: int, triangle_num: int = 0) -> np.ndarray:
     """Calculates which points are within a triangle in 2D.
 
-    Args:
+    arguments:
         points (np.ndarray): array of the points in 2D.
         triangle (np.ndarray): array of the triangle's vertices in 2D, shape (3, 2).
         points_xlen (int): the number of unique x coordinates.
         triangle_num (int): the triangle number, default is 0.
 
-    Returns:
+    returns:
         triangle_points (np.ndarray): 2D array containing only the points within the triangle,
             with each row being the triangle number, points y index, and points x index.
     """
@@ -707,13 +726,13 @@ def mesh_points_in_triangle(triangle: np.ndarray,
                             triangle_num: int = 0) -> np.ndarray:
     """Calculates which implicit mesh points are within a triangle in 2D for normalised triangle.
 
-    Args:
+    arguments:
         triangle (np.ndarray): array of the triangle's vertices in 2D, shape (3, 2).
         points_xlen (int): the number of unique x coordinates, starting at 0.0, spacing 1.0.
         points_ylen (int): the number of unique y coordinates, starting at 0.0, spacing 1.0.
         triangle_num (int): the triangle number, default is 0.
 
-    Returns:
+    returns:
         triangle_points (np.ndarray): 2D array containing only the points within the triangle,
             with each row being the triangle number, points y index, and points x index.
     """
@@ -732,35 +751,16 @@ def mesh_points_in_triangle(triangle: np.ndarray,
     return triangle_points
 
 
-def points_in_polygons_parallel(points: np.ndarray, polygons: np.ndarray, points_xlen: int) -> np.ndarray:
-    """Calculates which points are within which polygons in 2D.
-
-    Args:
-        points (np.ndarray): array of the points in 2D.
-        polygons (np.ndarray): array of each polygons' vertices in 2D.
-        points_xlen (int): the number of unique x coordinates.
-
-    Returns:
-        polygons_points (np.ndarray): 2D array containing only the points within each polygon,
-            with each row being the polygon number, points y index, and points x index.
-    """
-    args = [(points, polygons[polygon_num], points_xlen, polygon_num) for polygon_num in range(len(polygons))]
-    with m_p.Pool() as p:
-        r = p.starmap(points_in_polygon, args)
-
-    return np.vstack(r)
-
-
 @njit
 def points_in_polygons(points: np.ndarray, polygons: np.ndarray, points_xlen: int) -> np.ndarray:
     """Calculates which points are within which polygons in 2D.
 
-    Args:
+    arguments:
         points (np.ndarray): array of the points in 2D.
         polygons (np.ndarray): array of each polygons' vertices in 2D.
         points_xlen (int): the number of unique x coordinates.
 
-    Returns:
+    returns:
         polygons_points (np.ndarray): 2D array (list-like) containing only the points within each polygon,
             with each row being the polygon number, points y index, and points x index.
     """
@@ -776,12 +776,12 @@ def points_in_polygons(points: np.ndarray, polygons: np.ndarray, points_xlen: in
 def points_in_triangles_njit(points: np.ndarray, triangles: np.ndarray, points_xlen: int) -> np.ndarray:
     """Calculates which points are within which triangles in 2D.
 
-    Args:
+    arguments:
         points (np.ndarray): array of the points in 2D.
         triangles (np.ndarray): array of each triangles' vertices in 2D, shape (N, 3, 2).
         points_xlen (int): the number of unique x coordinates.
 
-    Returns:
+    returns:
         triangles_points (np.ndarray): 2D array (list-like) containing only the points within each triangle,
             with each row being the triangle number, points y index, and points x index.
     """
@@ -797,11 +797,11 @@ def points_in_triangles_njit(points: np.ndarray, triangles: np.ndarray, points_x
 def meshgrid(x: np.ndarray, y: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
     """Returns coordinate matrices from coordinate vectors x and y.
 
-    Args:
+    arguments:
         x (np.ndarray): 1d array of x coordinates.
         y (np.ndarray): 1d array of y coordinates.
 
-    Returns:
+    returns:
         Tuple containing:
 
             - xx (np.ndarray): the elements of x repeated to fill the matrix along the first dimension.
@@ -865,10 +865,10 @@ def points_in_triangles_aligned(nx: int, ny: int, dx: float, dy: float, triangle
 def triangle_box(triangle: np.ndarray) -> Tuple[float, float, float, float]:
     """Finds the minimum and maximum x and y values of a single traingle.
 
-    Args:
+    arguments:
         triangle (np.ndarray): array of the traingle's vertices' x and y coordinates.
 
-    Returns:
+    returns:
         Tuple containing:
 
             - (float): minimum x value.
@@ -887,20 +887,23 @@ def vertical_intercept(x: float, x_values: np.ndarray, y_values: np.ndarray) ->
     
     If the x value given is not within the x values of the points, returns None.
 
-    Args:
+    arguments:
         x (float): x value at which to determine the y value.
         x_values (np.ndarray): the x coordinates of point 1 and point 2.
         y_values (np.ndarray): the y coordinates of point 1 and point 2.
 
-    Returns:
+    returns:
         y (Optional[float]): y value of the straight line between point 1 and point 2,
             evaluated at x. If x is outside the x_values range, y is None.
     """
     y = None
     if x >= min(x_values) and x <= max(x_values):
-        m = (y_values[1] - y_values[0]) / (x_values[1] - x_values[0])
-        c = y_values[1] - m * x_values[1]
-        y = m * x + c
+        if x_values[0] == x_values[1]:
+            y = y_values[0]
+        else:
+            m = (y_values[1] - y_values[0]) / (x_values[1] - x_values[0])
+            c = y_values[1] - m * x_values[1]
+            y = m * x + c
     return y
 
 
@@ -908,35 +911,34 @@ def vertical_intercept(x: float, x_values: np.ndarray, y_values: np.ndarray) ->
 def points_in_triangles_aligned_optimised(nx: int, ny: int, dx: float, dy: float, triangles: np.ndarray) -> np.ndarray:
     """Calculates which points are within which triangles in 2D for a regular mesh of aligned points.
 
-    Args:
+    arguments:
         nx (int): number of points in x axis
         ny (int): number of points in y axis
         dx (float): spacing of points in x axis (first point is at half dx)
         dy (float): spacing of points in y axis (first point is at half dy)
-        triangles (np.ndarray): float array of each triangles' vertices in 2D, shape (N, 3, 2).
-        points_xlen (int): the number of unique x coordinates.
+        triangles (np.ndarray): float array of each triangles' vertices in 2D, shape (N, 3, 2)
 
-    Returns:
+    returns:
         triangles_points (np.ndarray): 2D array (list-like) containing only the points within each triangle,
-            with each row being the triangle number, points y index, and points x index.
+            with each row being the triangle number, points y index, and points x index
     """
-    grid_x = np.arange(dx / 2, dx / 2 + dx * nx, dx)
-    grid_y = np.arange(dy / 2, dy / 2 + dy * ny, dy)
+    grid_x = dx * (0.5 + np.arange(nx).astype(np.float64))
+    grid_y = dy * (0.5 + np.arange(ny).astype(np.float64))
     triangles_points_list = []
     for triangle_num in range(len(triangles)):
         triangle = triangles[triangle_num]
         min_x, max_x, min_y, max_y = triangle_box(triangle)
-        x_values = grid_x[np.logical_and(grid_x > min_x, grid_x < max_x)]
-        y_values = grid_y[np.logical_and(grid_y > min_y, grid_y < max_y)]
+        x_values = grid_x[np.logical_and(grid_x >= min_x, grid_x < max_x)]
+        y_values = grid_y[np.logical_and(grid_y >= min_y, grid_y < max_y)]
         for x in x_values:
             ys = []
             ys.append(vertical_intercept(x, triangle[1:, 0], triangle[1:, 1]))
             ys.append(vertical_intercept(x, triangle[:2, 0], triangle[:2, 1]))
             ys.append(vertical_intercept(x, triangle[::2, 0], triangle[::2, 1]))
-            ys = [x for x in ys if x is not None]
+            ys = [u for u in ys if u is not None]
             valid_y = y_values[np.logical_and(y_values >= min(ys), y_values <= max(ys))]
-            x_idx = int(x / dx - 0.5)
-            triangles_points_list.extend([[triangle_num, int(y / dy - 0.5), x_idx] for y in valid_y])
+            x_idx = int(x / dx)
+            triangles_points_list.extend([[triangle_num, int(y / dy), x_idx] for y in valid_y])
 
     if len(triangles_points_list) == 0:
         triangles_points = np.empty((0, 3), dtype = np.int32)
@@ -947,7 +949,12 @@ def points_in_triangles_aligned_optimised(nx: int, ny: int, dx: float, dy: float
 
 
 def triangle_normal_vector(p3):
-    """For a triangle in 3D space, defined by 3 vertex points, returns a unit vector normal to the plane of the triangle."""
+    """For a triangle in 3D space, defined by 3 vertex points, returns a unit vector normal to the plane of the triangle.
+
+    note:
+        resulting vector implicitly assumes that xy & z units are the same; if this is not the case, adjust vector
+        afterwards as required
+    """
 
     # todo: handle degenerate triangles
     return unit_vector(cross_product(p3[0] - p3[1], p3[0] - p3[2]))
@@ -955,7 +962,12 @@ def triangle_normal_vector(p3):
 
 @njit
 def triangle_normal_vector_numba(points):
-    """For a triangle in 3D space, defined by 3 vertex points, returns a unit vector normal to the plane of the triangle."""
+    """For a triangle in 3D space, defined by 3 vertex points, returns a unit vector normal to the plane of the triangle.
+
+    note:
+        resulting vector implicitly assumes that xy & z units are the same; if this is not the case, adjust vector
+        afterwards as required
+    """
     v = np.cross(points[0] - points[1], points[0] - points[2])
     return v / np.linalg.norm(v)
 
@@ -1092,6 +1104,29 @@ def clockwise_sorted_indices(p, b):
     return np.array([i for (_, i) in sorted(hull_list)], dtype = int)
 
 
+def xy_sorted(p, axis = None):
+    """Returns copy of points p sorted according to x or y (whichever has greater range).
+
+    arguments:
+        p (numpy float array of shape (..., 2) or (..., 3)): points to be sorted
+        axis (int, optional): 0 for x sort; 1 for y sort; None for whichever has greater range
+
+    returns:
+        p', axis where p' is a list-like (2D) version of p, sorted by either x or y and axis is 0
+        if the sort was by x, 1 if it were by y
+
+    note:
+       returned array is always 2D, ie. list of points
+    """
+    assert p.ndim >= 2 and p.shape[-1] >= 2
+    p = p.reshape((-1, p.shape[-1]))
+    xy_range = np.nanmax(p, axis = 0) - np.nanmin(p, axis = 0)
+    if axis is None:
+        axis = (1 if xy_range[1] > xy_range[0] else 0)
+    spi = np.argsort(p[:, axis])
+    return p[spi], axis
+
+
 def _projected_xyz_axes(projection):
     assert projection in ['xy', 'xz', 'yz'], f'invalid projection {projection}'
     a0 = 'xyz'.index(projection[0])
