@@ -4,18 +4,16 @@ The module includes a dictionary of nexus WELLSPEC column keywords, functionalit
 read WELLSPEC files and transform the well data into Pandas DataFrames.
 """
 
-version = "19th April 2022"
-
 # Nexus is a registered trademark of the Halliburton Company
 
 import logging
-from typing import Any, Dict, Tuple, Type, Optional, List, Union, TextIO
 
 log = logging.getLogger(__name__)
 
 import numpy as np
 import pandas as pd
 import datetime
+from typing import Any, Dict, Tuple, Type, Optional, List, Union, TextIO
 
 import resqpy.olio.keyword_files as kf
 
@@ -230,18 +228,17 @@ def length_unit_conversion_applicable(keyword):  # pragma: no cover
     return wellspec_dict[keyword][4]
 
 
-def load_wellspecs(
-    wellspec_file: str,
-    well: Optional[str] = None,
-    column_list: Union[List[str], None] = [],
-    keep_duplicate_cells: bool = False,
-    keep_null_columns: bool = True,
-    last_data_only: bool = True,
-    usa_date_format: bool = False,
-) -> Dict[str, Union[pd.DataFrame, None]]:
+def load_wellspecs(wellspec_file: str,
+                   well: Optional[str] = None,
+                   column_list: Union[List[str], None] = [],
+                   keep_duplicate_cells: bool = False,
+                   keep_null_columns: bool = True,
+                   last_data_only: bool = True,
+                   usa_date_format: bool = False,
+                   return_dates_list: bool = False):
     """Reads the Nexus wellspec file returning a dictionary of well name to pandas dataframe.
 
-    Args:
+    arguments:
         wellspec_file (str): file path of ascii input file containing wellspec keywords.
         well (str, optional): if present, only the data for the named well are loaded. If None, data
             for all wells are loaded.
@@ -258,10 +255,19 @@ def load_wellspecs(
             dataframe, otherwise all of the well data are used at different times.
         usa_date_format (bool): If True, wellspec file is expected to contain date formats in MM/DD/YYYY.
             if False, DD/MM/YYYY.
+        return_dates_list (bool, default False): if True, a sorted list of unique dates present in the
+            wellspec file is also returned, with dates in iso format
 
-    Returns:
-       well_dict (Dict[str, Union[pd.DataFrame, None]]): mapping each well name found in the
-          wellspec file to a dataframe containing the wellspec data.
+    returns:
+        well_dict (Dict[str, Union[pd.DataFrame, None]]): mapping each well name found in the
+            wellspec file to a dataframe containing the wellspec data
+        or (well_dict, dates_list): where dates list is a sorted list of all dates present in the
+            wellspec file (including those not relevant to a specific well), in iso format
+
+    note:
+        if return_dates_list is True, the returned list always contains all dates from the wellspec file
+        that applied to any entry, regardless of the well and last_data_only arguments; the dates list
+        will not include a null entry, even if there are wellspec data before the first timestamp
     """
     assert wellspec_file, "No wellspec file specified."
 
@@ -272,10 +278,16 @@ def load_wellspecs(
 
     well_dict = {}
     well_pointers = get_well_pointers(wellspec_file, usa_date_format = usa_date_format)
+    dates_list = None
+    if return_dates_list:
+        dates_list = _prepare_dates_list(well_pointers)
 
     if column_list is None:
         well_dict = dict.fromkeys(well_pointers, None)
-        return well_dict
+        if return_dates_list:
+            return (well_dict, dates_list)
+        else:
+            return well_dict
 
     with open(wellspec_file, "r") as file:
         if well:
@@ -306,7 +318,20 @@ def load_wellspecs(
                 if well_data is not None:
                     well_dict[well_name] = well_data
 
-    return well_dict
+    if return_dates_list:
+        return (well_dict, dates_list)
+    else:
+        return well_dict
+
+
+def _prepare_dates_list(well_pointers):
+    dates = []
+    for entries in well_pointers.values():
+        for _, date in entries:
+            if date and date not in dates:
+                dates.append(date)
+    dates.sort()
+    return dates
 
 
 def get_well_pointers(
@@ -316,14 +341,14 @@ def get_well_pointers(
 ) -> Dict[str, List[Tuple[int, Union[None, str]]]]:
     """Gets the file locations of each well in the wellspec file for optimised processing of the data.
 
-    Args:
+    arguments:
         wellspec_file (str): file path of ascii input file containing wellspec keywords.
         usa_date_format (bool): if True, the date taken from the wellspec file is in the format
             MM/DD/YYYY, otherwise it is in the format DD/MM/YYYY.
         no_date_replacement (datetime.date, optional): if there is no date given for a well, this
             date is used.
 
-    Returns:
+    returns:
         well_pointers (Dict[str, List[Tuple[int, None/str]]]): mapping each well name found in
             the wellspec file to a list of their file locations and dates as tuples. If there is no
             date before the well data in the file, the date is None. If there is a FileNotFoundError
@@ -403,7 +428,7 @@ def get_well_data(
 
     The pointer argument is used to go to the file location where the well dataset is located.
 
-    Args:
+    arguments:
         file (TextIO): the opened wellspec file object.
         well_name (str): name of the well.
         pointer (int): the file object's start position of the well data represented as number of
@@ -442,12 +467,13 @@ def get_well_data(
     all_null = True
     while True:
         kf.skip_comments(file)
-        if kf.blank_line(file):
-            break  # unclear from Nexus doc what marks end of table
-        if kf.specific_keyword_next(file, "WELLSPEC") or kf.specific_keyword_next(file, "WELLMOD"):
+        if (kf.specific_keyword_next(file, "WELLSPEC") or kf.specific_keyword_next(file, "WELLMOD") or
+                kf.specific_keyword_next(file, "TIME")):
             break
         line = kf.strip_trailing_comment(file.readline())
         words = line.split()
+        if len(words) == 0:
+            break  # end of file
         assert len(words) >= len(columns_present), f"Insufficient data in line of wellspec table {well_name} [{line}]."
         if selecting:
             for col_index, col in enumerate(column_list):
@@ -517,7 +543,7 @@ def get_all_well_data(
     This differs from the get_well_data function in that here multiple datasets for a well are
     combined into a single dataframe if they exist.
 
-    Args:
+    arguments:
         file (TextIO): the opened wellspec file object.
         well_name (str): name of the well.
         pointers (List[Tuple[int, None/str]]): a list of the file object's start position of the
@@ -537,7 +563,7 @@ def get_all_well_data(
         last_data_only (bool): If True, only the last entry of well data in the file are used in the
             dataframe, otherwise all of the well data are used at different times.
 
-    Returns:
+    returns:
         Pandas dataframe of all well data for a specific well name or None if all the data are NA.
     """
     if last_data_only:
