@@ -1,12 +1,12 @@
 """xml_et.py: Resqml xml element tree utilities module."""
 
-version = '30th November 2021'
-
 import logging
 
 log = logging.getLogger(__name__)
 
 import os
+from functools import lru_cache
+import uuid
 
 # import xml element tree parse method and classes here to allow single point for switching between lxml and etree
 # alternative to lxml.etree: xml.etree.ElementTree
@@ -21,7 +21,6 @@ from resqpy.olio.xml_namespaces import namespace as ns
 
 pretend_to_be_fesapi = False
 use_fesapi_quirks = True
-use_tabs = False
 
 if use_fesapi_quirks:
     null_xml_text = '\n'
@@ -38,24 +37,25 @@ def strip_path(full_path):
     return os.path.basename(full_path)
 
 
-def stripped_of_prefix(prefixed):
-    """Returns a simplified version of an xml tag or other element with any {xsd defining prefix} stripped off."""
+def stripped_of_prefix(s):
+    """Returns a simplified version of an xml tag or other str with any {xsd defining prefix} stripped off."""
 
-    if prefixed is None:
+    if s is None:
         return None
-    if '}' in prefixed:
-        return prefixed[prefixed.rfind('}') + 1:]
-    return prefixed[prefixed.rfind(':') + 1:]
+    p = s.rfind('}')
+    if p >= 0:
+        return s[p + 1:]
+    return s[s.rfind(':') + 1:]
 
 
+@lru_cache()
 def colon_prefixed(curly_prefixed):
     """Returns a version of an xml tag with {url} prefix replaced with nsi: equivalent; also returns the nsi prefix."""
-
     if not curly_prefixed:
         return None, None
     if curly_prefixed[0] != '{':
         colon = curly_prefixed.find(':')
-        if colon < 0:
+        if colon == -1:
             return curly_prefixed, None
         return curly_prefixed, curly_prefixed[:colon]
     pre_end = curly_prefixed.rfind('}')
@@ -66,13 +66,27 @@ def colon_prefixed(curly_prefixed):
     return pre_colon + ':' + curly_prefixed[pre_end + 1:], pre_colon
 
 
+@lru_cache()
+def match(xml_name, name):
+    """Returns True if the xml_name stripped of prefix matches name."""
+    i = len(xml_name) - len(name)
+    if i > 0:
+        ch = xml_name[i - 1]
+        if ch != '}' and ch != ':':
+            return False
+        return xml_name[i:] == name
+    elif i == 0:
+        return xml_name == name
+    return False
+
+
 def find_tag(root, tag_name, must_exist = False):
     """Finds the first child in xml node with a (prefix-stripped) tag matching given tag name."""
 
     if root is None:
         return None
     for child in root:
-        if stripped_of_prefix(child.tag) == tag_name:
+        if match(child.tag, tag_name):
             return child
     if must_exist:
         raise ValueError(f"Expected tag {tag_name} not found in root {root}")
@@ -85,7 +99,7 @@ def find_tag_text(root, tag_name, must_exist = False):
     if root is None:
         return None
     for child in root:
-        if stripped_of_prefix(child.tag) == tag_name:
+        if match(child.tag, tag_name):
             return node_text(child)
     if must_exist:
         raise ValueError(f"Expected tag {tag_name} not found in root {root}")
@@ -98,7 +112,7 @@ def find_tag_bool(root, tag_name, must_exist = False):
     if root is None:
         return None
     for child in root:
-        if stripped_of_prefix(child.tag) == tag_name:
+        if match(child.tag, tag_name):
             return node_bool(child)
     if must_exist:
         raise ValueError(f"Expected tag {tag_name} not found in root {root}")
@@ -111,7 +125,7 @@ def find_tag_int(root, tag_name, must_exist = False):
     if root is None:
         return None
     for child in root:
-        if stripped_of_prefix(child.tag) == tag_name:
+        if match(child.tag, tag_name):
             return node_int(child)
     if must_exist:
         raise ValueError(f"Expected tag {tag_name} not found in root {root}")
@@ -124,7 +138,7 @@ def find_tag_float(root, tag_name, must_exist = False):
     if root is None:
         return None
     for child in root:
-        if stripped_of_prefix(child.tag) == tag_name:
+        if match(child.tag, tag_name):
             return node_float(child)
     if must_exist:
         raise ValueError(f"Expected tag {tag_name} not found in root {root}")
@@ -165,8 +179,8 @@ def find_nested_tags_cast(root, tag_list, dtype = None):
 def find_nested_tags_text(root, tag_list):
     """Return stripped text of node at deepest level of xml hierarchy.
     
-    Args:
-        tag_list: list of tags in a nested xml hierarchy
+    arguments:
+        tag_list (list of str): list of tags in a nested xml hierarchy
     """
 
     node = find_nested_tags(root, tag_list)
@@ -176,8 +190,8 @@ def find_nested_tags_text(root, tag_list):
 def find_nested_tags_bool(root, tag_list):
     """Return stripped text of node at deepest level of xml hierarchy as a bool.
     
-    Args:
-        tag_list: list of tags in a nested xml hierarchy
+    arguments:
+        tag_list (list of str): list of tags in a nested xml hierarchy
     """
     node = find_nested_tags(root, tag_list)
     return node_bool(node)
@@ -186,8 +200,8 @@ def find_nested_tags_bool(root, tag_list):
 def find_nested_tags_int(root, tag_list):
     """Return stripped text of node at deepest level of xml hierarchy as an int.
     
-    Args:
-        tag_list: list of tags in a nested xml hierarchy
+    arguments:
+        tag_list (list of str): list of tags in a nested xml hierarchy
     """
     node = find_nested_tags(root, tag_list)
     return node_int(node)
@@ -196,8 +210,8 @@ def find_nested_tags_int(root, tag_list):
 def find_nested_tags_float(root, tag_list):
     """Return stripped text of node at deepest level of xml hierarchy as a float.
     
-    Args:
-        tag_list: list of tags in a nested xml hierarchy
+    arguments:
+        tag_list (list of str): list of tags in a nested xml hierarchy
     """
     node = find_nested_tags(root, tag_list)
     return node_float(node)
@@ -210,7 +224,7 @@ def count_tag(root, tag_name):
         return None
     count = 0
     for child in root:
-        if stripped_of_prefix(child.tag) == tag_name:
+        if match(child.tag, tag_name):
             count += 1
     return count
 
@@ -222,7 +236,7 @@ def list_of_tag(root, tag_name):
         return None
     results = []
     for child in root:
-        if stripped_of_prefix(child.tag) == tag_name:
+        if match(child.tag, tag_name):
             results.append(child)
     return results
 
@@ -234,7 +248,7 @@ def list_of_descendant_tag(root, tag_name):
         return None
     results = []
     for child in root.iterdescendants():
-        if stripped_of_prefix(child.tag) == tag_name:
+        if match(child.tag, tag_name):
             results.append(child)
     return results
 
@@ -243,13 +257,16 @@ def list_obj_references(root, skip_hdf5 = True):
     """Returns list of nodes of type DataObjectReference."""
 
     if root is None:
-        return None
-    results = []
-    if node_type(root) == 'DataObjectReference' and not (skip_hdf5 and stripped_of_prefix(root.tag) == 'HdfProxy'):
-        results.append(root)
+        return []
+    for v in root.attrib.values():
+        if match(v, 'DataObjectReference'):
+            if skip_hdf5 and match(root.tag, 'HdfProxy'):
+                return []
+            return [root]
+    refs = []
     for child in root:
-        results += list_obj_references(child, skip_hdf5 = skip_hdf5)
-    return results
+        refs += list_obj_references(child, skip_hdf5 = skip_hdf5)
+    return refs
 
 
 def cut_obj_references(root, uuids_to_be_cut):
@@ -304,21 +321,19 @@ def content_type(content_type_str):
 def node_type(node, is_rels = False, strip_obj = False):
     """Returns the type as held in attributes of xml node; defining authority is stripped out."""
 
-    result = None
     if node is None:
         return None
+    result = None
     if is_rels:
         if 'Type' not in node.attrib.keys():
             return None
         type_str = node.attrib['Type']
         result = type_str[type_str.rfind('/') + 1:]
     else:
-        for key in node.attrib.keys():
-            if stripped_of_prefix(key) == 'type':
-                #           type_str = node.attrib[key]
-                #           return type_str[type_str.rfind(':') + 1:]
-                type_str = stripped_of_prefix(node.attrib[key])
-                result = type_str
+        for key, value in node.attrib.items():
+            if match(key, 'type'):
+                result = stripped_of_prefix(value)
+                break
     if result and strip_obj and result.startswith('obj_'):
         result = result[4:]
     return result
@@ -403,17 +418,22 @@ def print_xml_tree(root,
     return line_count
 
 
-def uuid_in_part_name(part_name):
+def uuid_in_part_name(part_name, return_uuid_str = False):
     """Returns uuid as embedded in part name."""
 
     # This might not always work
     if part_name is None:
         return None
-    if part_name.endswith('.xml') and len(part_name) >= 40:
-        return bu.uuid_from_string(part_name[-40:-4])
-    elif part_name.endswith('.xml.rels') and len(part_name) >= 45:
-        return bu.uuid_from_string(part_name[-45:-9])
-    return None
+    hex = None
+    if part_name[-4:] == '.xml' and len(part_name) >= 40:
+        hex = part_name[-40:-4]
+    elif part_name[-9:] == '.xml.rels' and len(part_name) >= 45:
+        hex = part_name[-45:-9]
+    if hex is None:
+        return None
+    if return_uuid_str:
+        return hex
+    return uuid.UUID(hex)
 
 
 def part_name_for_object(obj_type, uuid, prefixed = False, epc_subdir = None):
@@ -453,9 +473,10 @@ def uuid_for_part_root(root):
 
     if root is None:
         return None
-    if 'uuid' not in root.attrib.keys():
+    uuid_str = root.attrib.get('uuid')
+    if not uuid_str:
         return None
-    return bu.uuid_from_string(root.attrib['uuid'])
+    return bu.uuid_from_string(uuid_str)
 
 
 def patch_uuid_in_part_root(root, uuid):
@@ -625,8 +646,8 @@ def xyz_handedness(xy_axes: str, z_inc_down: bool):
 def ijk_handedness(geom_node):
     """Returns ijk true handedness as 'left', 'right' or 'unknown'.
     
-    Args:
-        GridIsRightHanded node in grid geometry node.
+    arguments:
+        geom_node: GridIsRightHanded node in grid geometry node.
     """
 
     if geom_node is None:
@@ -653,38 +674,22 @@ def creation_date_for_node(node):
 
 def write_xml_node(xml_fp, root, level = 0, namespace_keys = []):
     """Recursively write an xml node to an open file; return number of nodes written."""
-
-    def _escaped_text(text):
-        d = {'<': '&lt;', '>': '&gt;', '&': '&amp;'}
-        esc = '<&>'
-        # todo: include quotes if needed
-        e = ''
-        for ch in str(text):
-            if ch in esc:
-                e += d[ch]
-            else:
-                e += ch
-        return e
-
     if root is None:
         return 0
-
     ns_keys = namespace_keys.copy()
 
-    type_attr = None
     tag, pre_colon = colon_prefixed(root.tag)
 
-    if pre_colon in ['content_types', 'rels']:
-        tag = tag[len(pre_colon) + 1:]
+    if pre_colon == 'content_types':
+        tag = tag[14:]
+        ct_special = True
+    elif pre_colon == 'rels':
+        tag = tag[5:]
         ct_special = True
     else:
         ct_special = False
 
-    if use_tabs:
-        line = (level * '\t')
-    else:
-        line = (3 * level * ' ')
-    line += '<' + tag  # todo: if any tags involve special characters, use _escaped_text(tag)
+    line = 3 * level * ' ' + '<' + tag  # todo: if any tags involve special characters, use _escaped_text(tag)
     if pre_colon and pre_colon not in ns_keys:
         line += ' xmlns'
         if not ct_special:
@@ -695,17 +700,17 @@ def write_xml_node(xml_fp, root, level = 0, namespace_keys = []):
     attrib_ns_list = []
     attrib_list = []
     type_pre_colon = None
-    for key in root.attrib.keys():
+    for key, val in root.attrib.items():
         colon_attrib_key, pre_colon_attrib = colon_prefixed(key)
         if pre_colon_attrib and pre_colon_attrib not in ns_keys:
             attrib_ns_list.append(pre_colon_attrib)
-        if stripped_of_prefix(key) == 'type':
-            type_attr, type_pre_colon = colon_prefixed(root.attrib[key])
+        if match(key, 'type'):
+            type_attr, type_pre_colon = colon_prefixed(val)
             attrib_list.append(colon_attrib_key + '="' + type_attr + '"')
-        elif ct_special and colon_attrib_key == 'PartName' and root.attrib[key].startswith('obj_'):
-            attrib_list.append(colon_attrib_key + '="/' + root.attrib[key] + '"')
+        elif ct_special and colon_attrib_key == 'PartName' and val.startswith('obj_'):
+            attrib_list.append(colon_attrib_key + '="/' + val + '"')
         else:
-            attrib_list.append(colon_attrib_key + '="' + root.attrib[key] + '"')
+            attrib_list.append(colon_attrib_key + '="' + val + '"')
 
     for attrib_ns in attrib_ns_list:
         line += ' xmlns:' + attrib_ns + '="' + ns[attrib_ns] + '"'
@@ -714,43 +719,33 @@ def write_xml_node(xml_fp, root, level = 0, namespace_keys = []):
         line += ' xmlns:' + type_pre_colon + '="' + ns[type_pre_colon] + '"'
         ns_keys.append(type_pre_colon)
 
-    for attrib in attrib_list:
-        line += ' ' + attrib
+    if attrib_list:
+        line += ' ' + ' '.join(attrib_list)
 
     node_count = 1
-
-    if ct_special and len(root) == 0:
-
+    len_root = len(root)
+    if ct_special and len_root == 0:
         line += '/>\n'
         xml_fp.write(line.encode())
-    #      print(line, end = '') # debug
-
     else:
-
         line += '>'
-
-        if root.text and not root.text.isspace():
-            line += _escaped_text(root.text)
-
+        text = root.text
+        if (not text or text.isspace()) and tag == 'Title':
+            text = 'untitled'
+        if text and not text.isspace():
+            line += text.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
         xml_fp.write(line.encode())
-        #      print(line, end = '') # debug
-        indentation = ''
 
-        if len(root):
+        indentation = ''
+        if len_root:
             xml_fp.write(b'\n')
-            #         print()  # debug
-            if use_tabs:
-                indentation = (level * '\t')
-            else:
-                indentation = (3 * level * ' ')
+            indentation = 3 * level * ' '
             for child in root:
                 node_count += write_xml_node(xml_fp, child, level = level + 1, namespace_keys = ns_keys)
 
         line = indentation + '</' + tag + '>\n'
         xml_fp.write(line.encode())
 
-    #      print(line, end = '') # debug
-
     return node_count
 
 
