@@ -1,35 +1,42 @@
 """Submodule containing functions for attribute extraction for a property collection."""
 
-version = '1st December 2021'
-
 # Nexus is a registered trademark of the Halliburton Company
 
 import logging
 
 log = logging.getLogger(__name__)
 
-import resqpy
 import numpy as np
 import numpy.ma as ma
+
+import resqpy
+import resqpy.property as rqp
 import resqpy.olio.xml_et as rqet
 import resqpy.olio.uuid as bu
-from .property_common import property_kind_and_facet_from_keyword
-from .property_kind import PropertyKind
+import resqpy.property.property_kind as rqpk
 
 
 def _min_max_of_cached_array(collection, cached_name, cached_array, null_value, discrete):
     collection.__dict__[cached_name] = cached_array
-    zorro = collection.masked_array(cached_array, exclude_value = null_value)
-    if (zorro is None or zorro.size == 0 or zorro.dtype in [bool, np.int8] or
-        (not discrete and np.all(np.isnan(zorro))) or (discrete and np.all(zorro == null_value))):
-        min_value = max_value = None
+    if discrete:
+        if null_value is None:
+            min_value = np.min(cached_array)
+            max_value = np.max(cached_array)
+        else:
+            zorro = collection.masked_array(cached_array, exclude_value = null_value)
+            min_value = np.min(zorro)
+            max_value = np.max(zorro)
+            if min_value is ma.masked:
+                min_value = None
+            if max_value is ma.masked:
+                max_value = None
     else:
-        min_value = np.nanmin(zorro)
-        max_value = np.nanmax(zorro)
-    if min_value is ma.masked or min_value == np.NaN:
-        min_value = None
-    if max_value is ma.masked or max_value == np.NaN:
-        max_value = None
+        min_value = np.nanmin(cached_array)
+        max_value = np.nanmax(cached_array)
+        if np.isnan(min_value):
+            min_value = None
+        if np.isnan(max_value):
+            max_value = None
     return min_value, max_value
 
 
@@ -42,10 +49,10 @@ def _get_property_kind_uuid(collection, property_kind_uuid, property_kind, uom,
                 break
         if property_kind_uuid is None:
             # create local property kind object and fetch uuid
-            lpk = PropertyKind(collection.model,
-                               title = property_kind,
-                               example_uom = uom,
-                               parent_property_kind = 'discrete' if discrete else 'continuous')
+            lpk = rqpk.PropertyKind(collection.model,
+                                    title = property_kind,
+                                    example_uom = uom,
+                                    parent_property_kind = 'discrete' if discrete else 'continuous')
             lpk.create_xml()
             property_kind_uuid = lpk.uuid
     return property_kind_uuid
@@ -205,7 +212,7 @@ def _get_perm_k_part(collection, perms, perm_k_mode, share_perm_parts, perm_i_pa
             elif perm_k_mode == 'ntg squared':
                 ntg = collection.cached_part_array_ref(ntg_part)
                 kv *= ntg * ntg
-        kv_collection = resqpy.property.PropertyCollection()
+        kv_collection = rqp.PropertyCollection()
         kv_collection.set_support(support_uuid = collection.support_uuid, model = collection.model)
         kv_collection.add_cached_array_to_imported_list(kv,
                                                         'derived from horizontal perm with mode ' + str(perm_k_mode),
