@@ -1,29 +1,24 @@
 """Class for a collection of grid properties"""
 
-version = '24th November 2021'
-
 # Nexus is a registered trademark of the Halliburton Company
 
 import logging
 
 log = logging.getLogger(__name__)
-log.debug('property.py version ' + version)
 
 import os
-
 import numpy as np
 
+import resqpy.property as rqp
 import resqpy.olio.ab_toolbox as abt
 import resqpy.olio.box_utilities as bxu
 import resqpy.olio.load_data as ld
 import resqpy.olio.write_data as wd
 import resqpy.olio.xml_et as rqet
-
-from .property_collection import PropertyCollection
-from .property_common import selective_version_of_collection
+import resqpy.property.property_common as rqp_c
 
 
-class GridPropertyCollection(PropertyCollection):
+class GridPropertyCollection(rqp.PropertyCollection):
     """Class for RESQML Property collection for an IJK Grid, inheriting from PropertyCollection."""
 
     def __init__(self, grid = None, property_set_root = None, realization = None):
@@ -878,7 +873,8 @@ def _add_to_imported(collection, a, title, info, null_value = None, const_value
         facet = info[9],
         realization = info[0],
         const_value = const_value,
-        points = info[21])
+        points = info[21],
+        time_series_uuid = info[11])
 
 
 def _extend_imported_initial_assertions(other, box, refinement, coarsening):
@@ -937,11 +933,15 @@ def _extend_imported_with_coarsening(collection, other, box, coarsening, realiza
 
 def _extend_imported_get_fine_collections(other, realization):
     # look for properties by kind, process in order: rock volume, net to gross ratio, porosity, permeability, saturation
-    source_rv = selective_version_of_collection(other, realization = realization, property_kind = 'rock volume')
-    source_ntg = selective_version_of_collection(other, realization = realization, property_kind = 'net to gross ratio')
-    source_poro = selective_version_of_collection(other, realization = realization, property_kind = 'porosity')
-    source_sat = selective_version_of_collection(other, realization = realization, property_kind = 'saturation')
-    source_perm = selective_version_of_collection(other, realization = realization, property_kind = 'permeability rock')
+    source_rv = rqp_c.selective_version_of_collection(other, realization = realization, property_kind = 'rock volume')
+    source_ntg = rqp_c.selective_version_of_collection(other,
+                                                       realization = realization,
+                                                       property_kind = 'net to gross ratio')
+    source_poro = rqp_c.selective_version_of_collection(other, realization = realization, property_kind = 'porosity')
+    source_sat = rqp_c.selective_version_of_collection(other, realization = realization, property_kind = 'saturation')
+    source_perm = rqp_c.selective_version_of_collection(other,
+                                                        realization = realization,
+                                                        property_kind = 'permeability rock')
     # todo: add kh and some other property kinds
 
     return source_rv, source_ntg, source_poro, source_sat, source_perm
@@ -1059,15 +1059,20 @@ def _extend_imported_coarsen_perm(source_perm, other, box, collection, realizati
 def _extend_imported_coarsen_lengths(other, box, collection, realization, uncache_other_arrays, coarsening,
                                      copy_all_realizations):
     # cell lengths
-    source_cell_lengths = selective_version_of_collection(other,
-                                                          realization = realization,
-                                                          property_kind = 'cell length')
+    source_cell_lengths = rqp_c.selective_version_of_collection(other,
+                                                                realization = realization,
+                                                                property_kind = 'cell length')
     for (part, info) in source_cell_lengths.dict.items():
         if not copy_all_realizations and info[0] != realization:
             continue
         fine_cl_array = _array_box(other, part, box = box, uncache_other_arrays = uncache_other_arrays)
-        assert info[5] == 1 and info[8] == 'direction'
-        axis = 'KJI'.index(info[9][0].upper())
+        assert info[5] == 1  # count
+        facet_type = info[8]
+        facet = info[9]
+        if not facet_type:
+            (_, facet_type, facet) = rqp.property_kind_and_facet_from_keyword(info[10])
+        assert facet_type == 'direction'
+        axis = 'KJI'.index(facet[0].upper())
         coarse_cl_array = _coarsening_sum(coarsening, fine_cl_array, axis = axis)
         _add_to_imported(collection, coarse_cl_array, 'coarsened from grid ' + str(other.support.uuid), info)
 
@@ -1099,7 +1104,7 @@ def _extend_imported_no_coarsening(collection, other, box, refinement, realizati
     if realization is None:
         source_collection = other
     else:
-        source_collection = selective_version_of_collection(other, realization = realization)
+        source_collection = rqp_c.selective_version_of_collection(other, realization = realization)
 
     for (part, info) in source_collection.dict.items():
         _extend_imported_no_coarsening_single(source_collection, part, info, collection, other, box, refinement,
@@ -1135,7 +1140,8 @@ def _extend_imported_no_coarsening_single(source_collection, part, info, collect
         facet = info[9],
         realization = info[0],
         const_value = const_value,
-        points = info[21])
+        points = info[21],
+        time_series_uuid = info[11])
 
 
 def _extend_imported_no_coarsening_single_resampling(a, info, collection, refinement):
