@@ -1,6 +1,4 @@
-"""_surface.py: surface class based on resqml standard."""
-
-version = '5th July 2022'
+"""Surface class based on RESQML TriangulatedSetRepresentation class."""
 
 # RMS and ROXAR are registered trademarks of Roxar Software Solutions AS, an Emerson company
 # GOCAD is also a trademark of Emerson
@@ -19,13 +17,16 @@ import resqpy.olio.vector_utilities as vec
 import resqpy.olio.write_hdf5 as rwh5
 import resqpy.olio.xml_et as rqet
 import resqpy.property as rqp
+import resqpy.surface as rqs
+import resqpy.surface._base_surface as rqsb
+import resqpy.surface._triangulated_patch as rqstp
+import resqpy.weights_and_measures as wam
+
 from resqpy.olio.xml_namespaces import curly_namespace as ns
 from resqpy.olio.zmap_reader import read_mesh
-from ._base_surface import BaseSurface
-from ._triangulated_patch import TriangulatedPatch
 
 
-class Surface(BaseSurface):
+class Surface(rqsb.BaseSurface):
     """Class for RESQML triangulated set surfaces."""
 
     resqml_type = 'TriangulatedSetRepresentation'
@@ -83,7 +84,7 @@ class Surface(BaseSurface):
            4. as a simple triangulation of a mesh in an ascii file
            5. from a GOCAD-TSurf format file
            5. as an empty surface
-           if an empty surface is created, 'set_from_...' methods are available to then set for one of:
+           if an empty surface is created, set_from... methods are available to then set for one of:
            - a horizontal plane
            - a single triangle
            - a 'sail' (a triangle wrapped onto a sphere)
@@ -103,7 +104,7 @@ class Surface(BaseSurface):
         self.points = None  # composite points (all patches)
         self.boundaries = None  # todo: read up on what this is for and look out for examples
         self.represented_interpretation_root = None
-        self.normal_vector = None  # a single derived vector that is roughly normal to the surface
+        self.normal_vector = None  # a single derived vector that is roughly (true) normal to the surface
         self.title = title
         super().__init__(model = parent_model,
                          uuid = uuid,
@@ -121,6 +122,20 @@ class Surface(BaseSurface):
         elif tsurf_file is not None:
             self.set_from_tsurf_file(tsurf_file)
 
+    @classmethod
+    def from_tri_mesh(cls, tri_mesh):
+        """Create a Surface from a TriMesh."""
+        assert isinstance(tri_mesh, rqs.TriMesh)
+        surf = cls(tri_mesh.model,
+                   crs_uuid = tri_mesh.crs_uuid,
+                   title = tri_mesh.title,
+                   surface_role = tri_mesh.surface_role,
+                   extra_metadata = tri_mesh.extra_metadata if hasattr(tri_mesh, 'extra_metadata') else None)
+        t, p = tri_mesh.triangles_and_points()
+        surf.set_from_triangles_and_points(t, p)
+        surf.represented_interpretation_root = tri_mesh.represented_interpretation_root
+        return surf
+
     def _load_from_xml(self):
         root_node = self.root
         assert root_node is not None
@@ -153,7 +168,7 @@ class Surface(BaseSurface):
                 continue
             patch_index = rqet.find_tag_int(child, 'PatchIndex')
             assert patch_index is not None
-            triangulated_patch = TriangulatedPatch(self.model, patch_index = patch_index, patch_node = child)
+            triangulated_patch = rqstp.TriangulatedPatch(self.model, patch_index = patch_index, patch_node = child)
             assert triangulated_patch is not None
             if self.crs_uuid is None:
                 self.crs_uuid = triangulated_patch.crs_uuid
@@ -269,7 +284,9 @@ class Surface(BaseSurface):
         self.crs_uuid = large_surface.crs_uuid
         self.patch_list = []
         for triangulated_patch in large_surface.patch_list:
-            trimmed_patch = TriangulatedPatch(self.model, patch_index = len(self.patch_list), crs_uuid = self.crs_uuid)
+            trimmed_patch = rqstp.TriangulatedPatch(self.model,
+                                                    patch_index = len(self.patch_list),
+                                                    crs_uuid = self.crs_uuid)
             trimmed_patch.set_to_trimmed_patch(triangulated_patch, xyz_box = box, xy_polygon = xy_polygon)
             if trimmed_patch is not None and trimmed_patch.triangle_count > 0:
                 self.patch_list.append(trimmed_patch)
@@ -339,17 +356,31 @@ class Surface(BaseSurface):
 
         triangles, _ = self.triangles_and_points()
         assert triangles is not None
-        tri_count = len(triangles)
-        all_edges = np.empty((tri_count, 3, 2))
-        for i in range(3):
-            all_edges[:, i, 0] = triangles[:, i - 1]
-            all_edges[:, i, 1] = triangles[:, i]
-        return np.unique(np.sort(all_edges.reshape((-1, 2)), axis = 1), axis = 0)
+        unique_edges, _ = triangulate.edges(triangles)
+        return unique_edges
+
+    def distinct_edges_and_counts(self):
+        """Returns unique edges as pairs of point indices, and a count of uses of each edge.
+
+        returns:
+            numpy int array of shape (N, 2), numpy int array of shape (N,)
+            where 2D array is list-like sorted points index pairs for unique edges
+            and 1D array contains corresponding edge usage count (usually 1 or 2)
+
+        notes:
+            first entry in each pair is always the lower of the two point indices;
+            for well formed surfaces, the count should everywhere be zero or one;
+            the function does not attempt to detect coincident points
+        """
+
+        triangles, _ = self.triangles_and_points()
+        assert triangles is not None
+        return triangulate.edges(triangles)
 
     def set_from_triangles_and_points(self, triangles, points):
         """Populate this (empty) Surface object from an array of triangle corner indices and an array of points."""
 
-        tri_patch = TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
+        tri_patch = rqstp.TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
         tri_patch.set_from_triangles_and_points(triangles, points)
         self.patch_list = [tri_patch]
         self.uuid = bu.new_uuid()
@@ -399,11 +430,18 @@ class Surface(BaseSurface):
            and radial distance arguments
         """
 
+        crs = rqc.Crs(self.model, uuid = point_set.crs_uuid)
         p = point_set.full_array_ref()
-        if reorient:
-            p_xy, self.normal_vector, reorient_matrix = triangulate.reorient(p, max_dip = reorient_max_dip)
+        if crs.xy_units == crs.z_units or not reorient:
+            unit_adjusted_p = p
         else:
-            p_xy = p
+            unit_adjusted_p = p.copy()
+            wam.convert_lengths(unit_adjusted_p[:, 2], crs.z_units, crs.xy_units)
+        if reorient:
+            p_xy, self.normal_vector, reorient_matrix = triangulate.reorient(unit_adjusted_p,
+                                                                             max_dip = reorient_max_dip)
+        else:
+            p_xy = unit_adjusted_p
         if extend_with_flange:
             flange_points = triangulate.surrounding_xy_ring(p_xy,
                                                             count = flange_point_count,
@@ -413,12 +451,12 @@ class Surface(BaseSurface):
             if reorient:
                 # reorient back extenstion points into original p space
                 flange_points_reverse_oriented = vec.rotate_array(reorient_matrix.T, flange_points)
-                p_e = np.concatenate((p, flange_points_reverse_oriented), axis = 0)
+                p_e = np.concatenate((unit_adjusted_p, flange_points_reverse_oriented), axis = 0)
             else:
                 p_e = p_xy_e
         else:
             p_xy_e = p_xy
-            p_e = p
+            p_e = unit_adjusted_p
         log.debug('number of points going into dt: ' + str(len(p_xy_e)))
         success = False
         try:
@@ -433,6 +471,8 @@ class Surface(BaseSurface):
         log.debug('number of triangles: ' + str(len(t)))
         if make_clockwise:
             triangulate.make_all_clockwise_xy(t, p_e)  # modifies t in situ
+        if crs.xy_units != crs.z_units and reorient:
+            wam.convert_lengths(p_e[:, 2], crs.xy_units, crs.z_units)
         self.crs_uuid = point_set.crs_uuid
         self.set_from_triangles_and_points(t, p_e)
         if extend_with_flange:
@@ -451,22 +491,35 @@ class Surface(BaseSurface):
 
         _, p = self.triangles_and_points()
         if reorient:
-            p_xy, self.normal_vector, _ = triangulate.reorient(p)
+            unit_adjusted_p = self.unit_adjusted_points()
+            p_xy, self.normal_vector, _ = triangulate.reorient(unit_adjusted_p)
         else:
             p_xy = p
         triangulate.make_all_clockwise_xy(self.triangles, p_xy)  # modifies t in situ
         # assert np.all(vec.clockwise_triangles(p_xy, self.triangles) >= 0.0)
 
+    def unit_adjusted_points(self):
+        """Returns cached points or copy thereof with z values adjusted to xy units."""
+
+        _, p = self.triangles_and_points()
+        crs = rqc.Crs(self.model, uuid = self.crs_uuid)
+        if crs.xy_units == self.z_units:
+            return p
+        unit_adjusted_p = p.copy()
+        wam.convert_lengths(unit_adjusted_p[:, 2], crs.z_units, crs.xy_units)
+        return unit_adjusted_p
+
     def normal(self):
         """Returns a vector that is roughly normal to the surface.
 
         notes:
            the result becomes more meaningless the less planar the surface is;
-           even for a parfectly planar surface, the result is approximate
+           even for a parfectly planar surface, the result is approximate;
+           true normal vector is found when xy & z units differ
         """
 
         if self.normal_vector is None:
-            _, p = self.triangles_and_points()
+            p = self.unit_adjusted_points()
             _, self.normal_vector, _ = triangulate.reorient(p)
         return self.normal_vector
 
@@ -483,7 +536,7 @@ class Surface(BaseSurface):
 
         mesh_shape = mesh_xyz.shape
         assert len(mesh_shape) == 3 and mesh_shape[2] == 3
-        tri_patch = TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
+        tri_patch = rqstp.TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
         tri_patch.set_from_irregular_mesh(mesh_xyz, quad_triangles = quad_triangles)
         self.patch_list = [tri_patch]
         self.uuid = bu.new_uuid()
@@ -497,7 +550,7 @@ class Surface(BaseSurface):
 
         mesh_shape = mesh_xyz.shape
         assert len(mesh_shape) == 3 and mesh_shape[2] == 3
-        tri_patch = TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
+        tri_patch = rqstp.TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
         tri_patch.set_from_sparse_mesh(mesh_xyz)
         self.patch_list = [tri_patch]
         self.uuid = bu.new_uuid()
@@ -528,7 +581,7 @@ class Surface(BaseSurface):
 
         mesh_shape = mesh_xyz.shape
         assert len(mesh_shape) == 5 and mesh_shape[2:] == (2, 2, 3)
-        tri_patch = TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
+        tri_patch = rqstp.TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
         tri_patch.set_from_torn_mesh(mesh_xyz, quad_triangles = quad_triangles)
         self.patch_list = [tri_patch]
         self.uuid = bu.new_uuid()
@@ -560,7 +613,7 @@ class Surface(BaseSurface):
         """Populates this (empty) surface to represent faces of a cell, from corner points of shape (2, 2, 2, 3)."""
 
         assert cp.size == 24
-        tri_patch = TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
+        tri_patch = rqstp.TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
         tri_patch.set_to_cell_faces_from_corner_points(cp, quad_triangles = quad_triangles)
         self.patch_list = [tri_patch]
         self.uuid = bu.new_uuid()
@@ -575,7 +628,7 @@ class Surface(BaseSurface):
         self.patch_list = []
         p_index = 0
         for cell_cp in cp:
-            tri_patch = TriangulatedPatch(self.model, patch_index = p_index, crs_uuid = self.crs_uuid)
+            tri_patch = rqstp.TriangulatedPatch(self.model, patch_index = p_index, crs_uuid = self.crs_uuid)
             tri_patch.set_to_cell_faces_from_corner_points(cell_cp, quad_triangles = quad_triangles)
             self.patch_list.append(tri_patch)
             p_index += 1
@@ -615,7 +668,7 @@ class Surface(BaseSurface):
         :meta common:
         """
 
-        tri_patch = TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
+        tri_patch = rqstp.TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
         tri_patch.set_to_horizontal_plane(depth, box_xyz, border = border, quad_triangles = quad_triangles)
         self.patch_list = [tri_patch]
         self.uuid = bu.new_uuid()
@@ -623,7 +676,7 @@ class Surface(BaseSurface):
     def set_to_triangle(self, corners):
         """Populate this (empty) surface with a patch of one triangle."""
 
-        tri_patch = TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
+        tri_patch = rqstp.TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
         tri_patch.set_to_triangle(corners)
         self.patch_list = [tri_patch]
         self.uuid = bu.new_uuid()
@@ -635,7 +688,7 @@ class Surface(BaseSurface):
             corners (numpy float array of shape [2, 2, 3] or [4, 3]): 4 corners in logical ordering
         """
 
-        tri_patch = TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
+        tri_patch = rqstp.TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
         tri_patch.set_to_triangle_pair(corners.reshape((4, 3)))
         self.patch_list = [tri_patch]
         self.uuid = bu.new_uuid()
@@ -643,7 +696,7 @@ class Surface(BaseSurface):
     def set_to_sail(self, n, centre, radius, azimuth, delta_theta):
         """Populate this (empty) surface with a patch representing a triangle wrapped on a sphere."""
 
-        tri_patch = TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
+        tri_patch = rqstp.TriangulatedPatch(self.model, patch_index = 0, crs_uuid = self.crs_uuid)
         tri_patch.set_to_sail(n, centre, radius, azimuth, delta_theta)
         self.patch_list = [tri_patch]
         self.uuid = bu.new_uuid()
@@ -733,17 +786,65 @@ class Surface(BaseSurface):
             return None
         return intersects[indices[0]]
 
+    def sample_z_at_xy_points(self, points, multiple_handling = 'any'):
+        """Returns interpolated z values for an array of xy values.
+
+        arguments:
+            points (numpy float array of shape (..., 2 or 3)): xy points to sample surface at (z values ignored)
+            multiple_handling (str, default 'any'): one of 'any', 'minimum', 'maximum', 'exception'
+
+        returns:
+            numpy float array of shape points.shape[:-1] being z values interpolated from the surface z values
+
+        notes:
+            points must be in the same crs as the surface;
+            NaN will be set for any points that do not intersect with the surface in the xy projection;
+            multiple_handling argument controls behaviour when one sample point intersects surface more than
+            once: 'any' a random one of the intersection z values is returned; 'minimum' or 'maximum': the
+            numerical min or max of the z values is returned; 'exception': a ValueError is raised
+        """
+
+        assert points.ndim > 1 and 2 <= points.shape[-1] <= 3
+        assert multiple_handling in ['any', 'minimum', 'maximum', 'exception'],  \
+            f'invalid multiple handling mode: {multiple_handling}'
+        if points.shape[-1] == 3:
+            sample_xy = points.reshape((-1, 3))
+        else:
+            sample_xy = np.zeros((points.size // 2, 3), dtype = float)
+            sample_xy[:, :2] = points.reshape((-1, 2))
+        t, p = self.triangles_and_points()
+        p_list = vec.points_in_triangles_njit(sample_xy, p[t], 1)
+        vertical = np.array((0.0, 0.0, 1.0), dtype = float)
+        z = np.full(sample_xy.shape[0], np.NaN, dtype = float)
+        for triangle_index, p_index, _ in p_list:
+            # todo: replace following with cheaper trilinear interpolation, given vertical intersection line
+            xyz = meet.line_triangle_intersect_numba(sample_xy[p_index], vertical, p[t[triangle_index]], t_tol = 0.05)
+            if np.isnan(z[p_index]) or multiple_handling == 'any':
+                z[p_index] = xyz[2]
+            elif multiple_handling == 'minimum':
+                if xyz[2] < z[p_index]:
+                    z[p_index] = xyz[2]
+            elif multiple_handling == 'maximum':
+                if xyz[2] > z[p_index]:
+                    z[p_index] = xyz[2]
+            else:
+                raise ValueError(f'multiple {self.title} surface intersections at xy: {sample_xy[p_index]}')
+        return z.reshape(points.shape[:-1])
+
     def normal_vectors(self, add_as_property: bool = False) -> np.ndarray:
         """Returns the normal vectors for each triangle in the surface.
         
-        Args:
+        arguments:
             add_as_property (bool): if True, face_surface_normal_vectors_array is added as a property to the model.
 
-        Returns:
+        returns:
             normal_vectors_array (np.ndarray): the normal vectors corresponding to each triangle in the surface.
         """
         crs = rqc.Crs(self.model, uuid = self.crs_uuid)
         triangles, points = self.triangles_and_points()
+        if crs.xy_units != crs.z_units:
+            points = points.copy()
+            wam.convert_lengths(points[:, 2], crs.z_units, crs.xy_units)
         n_triangles = len(triangles)
         normal_vectors_array = np.empty((n_triangles, 3))
         for triangle_num in range(n_triangles):
@@ -755,17 +856,50 @@ class Surface(BaseSurface):
             pc = rqp.PropertyCollection()
             pc.set_support(support = self)
             crs = rqc.Crs(self.model, uuid = self.crs_uuid)
-            pc.add_cached_array_to_imported_list(normal_vectors_array,
-                                                 "computed from surface",
-                                                 "normal vector",
-                                                 uom = crs.xy_units,
-                                                 property_kind = "normal vector",
-                                                 indexable_element = "faces",
-                                                 points = True)
+            pc.add_cached_array_to_imported_list(
+                normal_vectors_array,
+                "computed from surface",
+                "normal vector",
+                uom = None,  # uom not used for points properties
+                property_kind = "normal vector",
+                indexable_element = "faces",
+                points = True)
             pc.write_hdf5_for_imported_list()
             pc.create_xml_for_imported_list_and_add_parts_to_model(find_local_property_kinds = True)
         return normal_vectors_array
 
+    def axial_edge_crossings(self, axis, value = 0.0):
+        """Returns list-like array of points interpolated from triangle edges that cross value in axis.
+
+        arguments:
+            axis (int): xyz axis; 0 for crossings in x axis, 1 for y, 2 for z
+            value (float, default 0.0): the value in axis at which crossing points are sought
+
+        returns:
+            numpy float array of shape (N, 3) being interpolated triangle edge points with the crossing
+                value in axis; or None if no crossings found
+        """
+
+        t, p = self.triangles_and_points()
+        zero = p[:, axis] - value  # +ve one side of value, -ve the other side
+        abs_zero = np.expand_dims(np.abs(zero), axis = -1)  # used to weight ends of crossing edges to interpolate
+        e, _ = triangulate.edges(t)  # list-like pairs of p indices for ends of distinct edges
+
+        # find crossing edges
+        crossing = np.where(zero[e[:, 0]] * zero[e[:, 1]] < 0.0)  # zero crossing edge indices in e
+
+        if not len(crossing[0]):
+            return None
+
+        ec = e[crossing]
+
+        s = abs_zero[ec[:, 0]] + abs_zero[ec[:, 1]]
+        # TODO: ignore divide by zero
+        ep = (p[ec[:, 0]] * abs_zero[ec[:, 1]] + p[ec[:, 1]] * abs_zero[ec[:, 0]]) / s
+        # TODO: unignore, and use midpoint of those edges?
+
+        return ep
+
     def write_hdf5(self, file_name = None, mode = 'a'):
         """Create or append to an hdf5 file, writing datasets for the triangulated patches after caching arrays.
 
