@@ -6,16 +6,19 @@ log = logging.getLogger(__name__)
 
 import datetime as dt
 import warnings
-from ._any_time_series import AnyTimeSeries
-from ._time_duration import TimeDuration
+
+import resqpy.time_series
+import resqpy.time_series._any_time_series as ats
+import resqpy.time_series._time_duration as td
 
 
-class TimeSeries(AnyTimeSeries):
+class TimeSeries(ats.AnyTimeSeries):
     """Class for RESQML Time Series without year offsets.
 
     notes:
-       use this class for time series on a human timeframe; use the resqpy GeologicTimeSeries class
-       instead if the time series is on a geological timeframe
+       individual RESQML timestamps are strings formatted in accordance with ISO 8601;
+       use this class for time series on a human timeframe;
+       use the resqpy GeologicTimeSeries class instead if the time series is on a geological timeframe
     """
 
     def __init__(self,
@@ -34,8 +37,9 @@ class TimeSeries(AnyTimeSeries):
         arguments:
            parent_model (model.Model): the resqpy model to which the time series will belong
            uuid (uuid.UUID, optional): the uuid of a TimeSeries object to be loaded from xml
-           first_time_stamp (str, optional): the first timestamp (in RESQML format) if not loading from xml;
-              this and the remaining arguments are ignored if loading from xml
+           first_timestamp (str, optional): the first timestamp (in RESQML format) if not loading from xml;
+              this and the remaining arguments are ignored if loading from xml; if present, timestamp must
+              be in ISO 8601 format, eg '2023-01-31' or '2023-01-31T13:30:00Z' or '2023-01-31T13:30:00.912'
            daily (non-negative int, optional): the number of one day interval timesteps to start the series
            monthly (non-negative int, optional): the number of 30 day interval timesteps to follow the daily
               timesteps
@@ -63,6 +67,7 @@ class TimeSeries(AnyTimeSeries):
         self.timeframe = 'human'
         self.timestamps = []  # ordered list of timestamp strings in resqml/iso format
         if first_timestamp is not None:
+            check_timestamp(first_timestamp)
             self.timestamps.append(first_timestamp)  # todo: check format of first_timestamp
             if daily is not None:
                 for _ in range(daily):
@@ -90,10 +95,10 @@ class TimeSeries(AnyTimeSeries):
         super_equivalence = super().is_equivalent(other_ts)
         if super_equivalence is not None:
             return super_equivalence
-        tolerance = TimeDuration(seconds = tol_seconds)
+        tolerance = td.TimeDuration(seconds = tol_seconds)
         for t_index in range(self.number_of_timestamps()):
-            diff = TimeDuration(earlier_timestamp = self.timestamps[t_index],
-                                later_timestamp = other_ts.timestamps[t_index])
+            diff = td.TimeDuration(earlier_timestamp = self.timestamps[t_index],
+                                   later_timestamp = other_ts.timestamps[t_index])
             if abs(diff.duration) > tolerance.duration:
                 return False
         return True
@@ -103,6 +108,7 @@ class TimeSeries(AnyTimeSeries):
 
         :meta common:
         """
+        check_timestamp(timestamp)
         index = len(self.timestamps) - 1
         while (index >= 0) and (self.timestamps[index] > timestamp):
             index -= 1
@@ -115,6 +121,7 @@ class TimeSeries(AnyTimeSeries):
 
         :meta common:
         """
+        check_timestamp(timestamp)
         index = 0
         while (index < len(self.timestamps)) and (self.timestamps[index] < timestamp):
             index += 1
@@ -127,6 +134,7 @@ class TimeSeries(AnyTimeSeries):
 
         :meta common:
         """
+        check_timestamp(timestamp)
         if not self.timestamps:
             return None
         before = self.index_for_timestamp_not_later_than(timestamp)
@@ -135,8 +143,8 @@ class TimeSeries(AnyTimeSeries):
         if before == len(self.timestamps) - 1 or self.timestamps[before] == timestamp:
             return before
         after = before + 1
-        early_delta = TimeDuration(earlier_timestamp = self.timestamps[before], later_timestamp = timestamp)
-        later_delta = TimeDuration(earlier_timestamp = timestamp, later_timestamp = self.timestamps[after])
+        early_delta = td.TimeDuration(earlier_timestamp = self.timestamps[before], later_timestamp = timestamp)
+        later_delta = td.TimeDuration(earlier_timestamp = timestamp, later_timestamp = self.timestamps[after])
         return before if early_delta.duration <= later_delta.duration else after
 
     def duration_between_timestamps(self, earlier_index, later_index):
@@ -146,8 +154,8 @@ class TimeSeries(AnyTimeSeries):
         """
         if earlier_index < 0 or later_index >= len(self.timestamps) or later_index < earlier_index:
             return None
-        return TimeDuration(earlier_timestamp = self.timestamps[earlier_index],
-                            later_timestamp = self.timestamps[later_index])
+        return td.TimeDuration(earlier_timestamp = self.timestamps[earlier_index],
+                               later_timestamp = self.timestamps[later_index])
 
     def days_between_timestamps(self, earlier_index, later_index):
         """Returns the number of whole days between a pair of timestamps, as an integer."""
@@ -189,7 +197,7 @@ class TimeSeries(AnyTimeSeries):
     # Could check for relationships involving the time series and disallow changes if any found?
     def add_timestamp(self, new_timestamp, allow_insertion = False):
         """Inserts a new timestamp into the time series."""
-        # todo: check that new_timestamp is in valid format (iso format + 'Z')
+        check_timestamp(new_timestamp)
         if allow_insertion:
             # NB: This can insert a timestamp anywhere in the series, will invalidate indices, possibly corrupting model
             index = self.index_for_timestamp_not_later_than(new_timestamp)
@@ -212,9 +220,16 @@ class TimeSeries(AnyTimeSeries):
 
     def extend_by_days(self, days):
         """Adds a timestamp to the end of the series, at a duration of days beyond the last timestamp."""
-        duration = TimeDuration(days = days)
+        duration = td.TimeDuration(days = days)
         self.extend_by_duration(duration)
 
     def datetimes(self):
         """Returns the timestamps as a list of python-datetime objects."""
         return [dt.datetime.fromisoformat(t.rstrip('Z')) for t in self.timestamps]
+
+
+def check_timestamp(timestamp):
+    """Check format of timestamp and raise ValueError if badly formed."""
+    if timestamp.endswith('Z'):
+        timestamp = timestamp[:-1]
+    _ = dt.datetime.fromisoformat(timestamp)
